import React from "react";
import logo from "./logo.svg";
import { Form, Button, Container, Row, Spinner, Alert } from "react-bootstrap";

class SignIn extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: "",
      password: "",
      noExpire: false,
      isLoading: false,
      badAuth: false,
      error: null,
    };

    this.onFormSubmit = this.onFormSubmit.bind(this);
  }

  onFormSubmit(e) {
    e.preventDefault();

    this.setState({ isLoading: true, error: null });
    const body = JSON.stringify({
      name: this.state.name,
      password: this.state.password,
      noExpire: this.state.noExpire,
    });

    fetch("/api/signin", { method: "POST", body })
      .then((response) => {
        if (response.status === 400) {
          var error = new Error("Bad auth");
          error.name = "bad-auth";
          throw error;
        } else if (!response.ok) {
          throw new Error(
            response.status.toString() + " " + response.statusText
          );
        }
        return response.json();
      })
      .then((data) => {
        this.setState({ isLoading: false, error: null, badAuth: false });
        this.props.onLogin(data.token, data.member);
      })
      .catch((error) => {
        if (error.name === "bad-auth") {
          this.setState({ isLoading: false, error: null, badAuth: true });
        } else {
          this.setState({ isLoading: false, error: error.message });
        }
      });
  }

  render() {
    let form = (
      <Form onSubmit={this.onFormSubmit}>
        <Form.Group>
          <Form.Label>Nombre</Form.Label>
          <Form.Control
            placeholder="Nombre"
            value={this.state.name}
            onChange={(e) => this.setState({ name: e.target.value })}
            isInvalid={this.state.badAuth}
          />
        </Form.Group>

        <Form.Group>
          <Form.Label>Contraseña</Form.Label>
          <Form.Control
            type="password"
            placeholder="Contraseña"
            value={this.state.password}
            onChange={(e) => this.setState({ password: e.target.value })}
            isInvalid={this.state.badAuth}
          />
          <Form.Control.Feedback type="invalid">
            Nombre o contraseña invalidos.
          </Form.Control.Feedback>
        </Form.Group>

        <Form.Group>
          <Form.Check
            type="switch"
            id="noExpire"
            label="Permanecer conectada con este navegador"
            onChange={(e) => this.setState({ noExpire: e.target.checked })}
          />
        </Form.Group>

        <Button variant="primary" type="submit" className="w-100 mt-3">
          Entra
        </Button>
      </Form>
    );

    if (this.state.isLoading) {
      form = <Spinner animation="border" />;
    }

    let alert;
    if (this.state.error != null) {
      alert = (
        <Alert variant="danger">
          Ha ocurrido un error iniciando sesión: {this.state.error}
        </Alert>
      );
    }

    return (
      <Container>
        <Row className="justify-content-center">
          <img src={logo} alt="Garbanzo Negro" />
        </Row>
        {alert}
        <Row className="justify-content-center">{form}</Row>
      </Container>
    );
  }
}

export default SignIn;
