import React from "react";
import { Redirect } from "react-router-dom";
import {
  Container,
  Form,
  Row,
  Col,
  Button,
  Alert,
  Spinner,
} from "react-bootstrap";
import ProductPicker from "../ProductPicker";
import AuthContext from "../AuthContext";

function date2string(date) {
  return date.toISOString().split("T")[0];
}

function daysAfterNow(days) {
  let date = new Date();
  date.setDate(date.getDate() + days);
  return date;
}

class CreateOrder extends React.Component {
  static contextType = AuthContext;

  constructor(props) {
    super(props);
    this.state = {
      products: [],
      name: "",
      description: "",
      deadline: daysAfterNow(3),
      orderId: null,
      isLoading: false,
      error: null,
    };
  }

  submit(e) {
    e.preventDefault();

    this.setState({ isLoading: true, error: null, noMoney: false });
    const products = this.state.products.map((p) => {
      return { code: p.code };
    });
    const body = JSON.stringify({
      name: this.state.name,
      description: this.state.description,
      deadline: this.state.deadline,
      products,
    });
    fetch("/api/order", {
      headers: { "x-authentication": this.context.token },
      method: "POST",
      body,
    })
      .then((response) => {
        if (!response.ok) {
          throw new Error(
            response.status.toString() + " " + response.statusText
          );
        }
        return response.json();
      })
      .then((o) => this.setState({ orderId: o.ID, isLoading: false }))
      .catch((error) =>
        this.setState({ isLoading: false, error: error.message })
      );
  }

  render() {
    if (this.state.isLoading) {
      return <Spinner animation="border" />;
    }

    let alert;
    if (this.state.error != null) {
      alert = (
        <Alert variant="danger">
          Ha ocurrido un error creando el pedido: {this.state.error}
        </Alert>
      );
    }
    if (this.state.orderId !== null) {
      return <Redirect to={"/order/" + this.state.orderId} />;
    }

    return (
      <Container>
        {alert}
        <Form onSubmit={(e) => this.submit(e)}>
          <Form.Group as={Row}>
            <Form.Label as="legend" column sm={3}>
              Nombre
            </Form.Label>
            <Col sm={9}>
              <Form.Control
                placeholder="nombre del pedido"
                value={this.state.name}
                onChange={(e) => this.setState({ name: e.target.value })}
              />
            </Col>
          </Form.Group>
          <Form.Group as={Row}>
            <Form.Label as="legend" column sm={3}>
              Descripción
            </Form.Label>
            <Col sm={9}>
              <Form.Control
                as="textarea"
                value={this.state.description}
                onChange={(e) => this.setState({ description: e.target.value })}
              />
            </Col>
          </Form.Group>
          <Form.Group as={Row}>
            <Form.Label as="legend" column sm={3}>
              Fecha límite
            </Form.Label>
            <Col sm={9}>
              <Form.Control
                type="date"
                value={date2string(this.state.deadline)}
                onChange={(e) =>
                  this.setState({ deadline: Date(e.target.value) })
                }
                min={date2string(daysAfterNow(1))}
              />
            </Col>
          </Form.Group>
          <ProductPicker
            picks={this.state.products}
            setPicks={(products) => this.setState({ products })}
          />
          <Form.Group as={Row}>
            <Col>
              <div className="text-right">
                <Button type="submit">Abrir pedido</Button>
              </div>
            </Col>
          </Form.Group>
        </Form>
      </Container>
    );
  }
}

export default CreateOrder;
