import React from "react";
import {
  Alert,
  Spinner,
  Form,
  InputGroup,
  Button,
  Col,
  Row,
} from "react-bootstrap";
import AuthContext from "../AuthContext";
import { printMoney } from "../util";

class PurchaseOrder extends React.Component {
  static contextType = AuthContext;

  constructor(props) {
    super(props);
    let order = props.order.products;
    order = order.map((p) => {
      p.amount = 0;
      p.purchased = 0;
      return p;
    });
    props.order.transactions.forEach((t) => {
      t.order_purchase.forEach((purchase) => {
        const i = order.findIndex((p) => p.code === purchase.product_code);
        if (i) {
          order[i].purchased += purchase.amount;
        }
      });
    });
    this.state = {
      order: order,
      total: 0,
      isLoading: false,
      noMoney: false,
      error: null,
    };
  }

  send(e) {
    e.preventDefault();

    this.setState({ isLoading: true, error: null, noMoney: false });
    const purchase = this.state.order.map((p) => {
      return {
        product_code: p.code,
        amount: parseInt(p.amount),
      };
    });
    const body = JSON.stringify({
      order: this.props.order.ID,
      purchase,
    });
    console.log(body);
    fetch("/api/order/purchase", {
      headers: { "x-authentication": this.context.token },
      method: "POST",
      body,
    }).then((response) => {
      if (response.status === 400) {
        this.setState({ isLoading: false, noMoney: true });
      } else if (!response.ok) {
        this.setState({
          isLoading: false,
          error: response.status.toString() + " " + response.statusText,
        });
      } else {
        this.props.onSend(this.state.order, this.state.total);
      }
    });
  }

  setAmount(index, amount) {
    let order = this.state.order;
    order[index].amount = amount;

    const add = (acc, p) => acc + p.price * p.amount;
    const total = order.reduce(add, 0);

    this.setState({ order, total });
  }

  render() {
    if (this.state.isLoading) {
      return <Spinner animation="border" />;
    }

    let alert;
    if (this.state.noMoney) {
      alert = (
        <Alert variant="warning">
          No tienes suficiente dinero para realizar este pedido.
        </Alert>
      );
    } else if (this.state.error != null) {
      alert = (
        <Alert variant="danger">
          Ha ocurrido un error enviando el pedido: {this.state.error}
        </Alert>
      );
    }

    const formEntries = this.state.order.map((p, i) => (
      <Form.Group key={p.code} as={Row}>
        <Form.Label column className="text-right">
          {p.name} ({p.code}):
        </Form.Label>
        <Col>
          <InputGroup>
            <Form.Control
              type="number"
              min="0"
              placeholder="cantidad"
              value={p.amount}
              onChange={(e) => this.setAmount(i, e.target.value)}
            />
            <InputGroup.Append>
              <InputGroup.Text>{printMoney(p.price)}€</InputGroup.Text>
            </InputGroup.Append>
          </InputGroup>
        </Col>
      </Form.Group>
    ));

    return (
      <Form onSubmit={(e) => this.send(e)}>
        {alert}
        {formEntries}
        <Form.Group as={Row}>
          <Col className="text-right">
            <Button type="submit">Realizar pedido</Button>
          </Col>
          <Col>
            <h3>Total: {printMoney(this.state.total)}€</h3>
          </Col>
        </Form.Group>
      </Form>
    );
  }
}

export default PurchaseOrder;
