import React from "react";
import Fetcher from "../Fetcher";
import { Container, Row, Col, Badge } from "react-bootstrap";
import PurchaseOrder from "./PurchaseOrder";
import { printDate } from "../util";
import AuthContext from "../AuthContext";
import { printMoney } from "../util";

function ShowOrderTransaction(props) {
  const list = props.transaction.order_purchase.map((o) => (
    <li key={o.product.code}>
      {o.product.name} ({o.product.code}): {o.amount}
    </li>
  ));
  return (
    <div>
      <h3>Mi pedido</h3>
      <ul>{list}</ul>
      <p>Total: {printMoney(-props.transaction.total)} €</p>
    </div>
  );
}

function ShowOrderResults(props) {
  let products = props.order.products.map((p) => {
    p.amount = 0;
    return p;
  });
  const transactions = props.order.transactions.map((t) => {
    const list = t.order_purchase.map((purchase) => {
      const i = products.findIndex((p) => p.code === purchase.product_code);
      if (!i) {
        return null;
      }
      products[i].amount += purchase.amount;
      if (purchase.amount) {
        const key =
          t.member.num.toString() + "-" + purchase.product_code.toString();
        return (
          <li key={key}>
            {products[i].name} {purchase.amount}
          </li>
        );
      }
      return null;
    });
    return (
      <li key={t.member.num}>
        {t.member.name} ({t.member.num}):
        <ul>{list}</ul>
      </li>
    );
  });

  const product_list = products.map((p) => (
    <li key={p.code}>
      {p.name}: {p.amount}
    </li>
  ));
  return (
    <div>
      <h3>Productos pedidos</h3>
      <ul>{product_list}</ul>

      <h3>Pedidos</h3>
      <ul>{transactions}</ul>
    </div>
  );
}

class ShowOrder extends React.Component {
  static contextType = AuthContext;

  constructor(props) {
    super(props);
    this.state = {
      order: {
        products: [],
        transactions: [],
      },
      transaction: null,
    };
  }

  onSend(purchase, total) {
    const order_purchase = purchase.map((p) => {
      p.product = p;
      return p;
    });
    const transaction = { order_purchase, total: -total };
    this.setState({ transaction });
  }

  showTransaction() {
    if (this.state.transaction) {
      return (
        <ShowOrderTransaction
          order={this.state.order}
          transaction={this.state.transaction}
        />
      );
    }
    if (this.state.order.active) {
      return (
        <PurchaseOrder
          order={this.state.order}
          onSend={(p, t) => this.onSend(p, t)}
        />
      );
    }
  }

  setData(data) {
    this.setState({ order: data.order, transaction: data.transaction });
  }

  render() {
    let expired;
    if (!this.state.order.active) {
      expired = <Badge variant="info">finalizado</Badge>;
    }

    const { id } = this.props.match.params;
    return (
      <Container>
        <Fetcher
          url={"/api/order/" + id}
          onFetch={(data) => this.setData(data)}
        >
          <Row>
            <Col>
              <h1>{this.state.order.name}</h1>
            </Col>
            <Col>
              <p className="text-right">
                Fecha limite: {printDate(this.state.order.deadline)}
                {expired}
              </p>
            </Col>
          </Row>
          <p>{this.state.order.description}</p>

          {this.showTransaction()}
          <ShowOrderResults order={this.state.order} />
        </Fetcher>
      </Container>
    );
  }
}

export default ShowOrder;
